@@ -26,8 +26,14 @@ logging.Formatter.converter = time.gmtime
 LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO").upper()
 LOG_FORMAT = "[%(levelname)s]  %(asctime)s.%(msecs)03dZ  %(name)s:%(funcName)s:%(lineno)d  %(message)s"
 LOG_FORMATTER = logging.Formatter(LOG_FORMAT, "%Y-%m-%dT%H:%M:%S")
-handler = logging.StreamHandler(sys.stdout)
-handler.formatter = LOG_FORMATTER
-LOGGER = logging.getLogger("aio-aws")
-LOGGER.addHandler(handler)
-LOGGER.setLevel(LOG_LEVEL)
+HANDLER = logging.StreamHandler(sys.stdout)
+HANDLER.formatter = LOG_FORMATTER
+
+
+def get_logger(name: str = "aio-aws") -> logging.Logger:
+    logger = logging.getLogger(name)
+    if not any(isinstance(h, logging.StreamHandler) for h in logger.handlers):
+        logger.addHandler(HANDLER)
+    logger.setLevel(LOG_LEVEL)
+    logger.propagate = False
+    return logger
